/*DOUBLE GLM MACRO*/

%macro doubleglm(base=, y=, x=,z=, dist=normal, link=identity, intercept=y,scale=deviance, 
dist_disp=gamma, link_disp=log, intercept_disp=y, alpha=0.05, 
scale_disp=deviance, maxiter=100, eps=0.000001, maxit=50);
/**************************************************/
/*DIST = GAMMA, NORMAL, POISSON, IG, BINOMIAL, NB */
/*LINK = INV, LOG, IDENTITY, INV2, LOGIT          */
/*INTERCEPT = y(YES), n(NO)                      */
/*SCALE = DEVIANCE, PEARSON                       */
/*DIST_DISP = GAMMA, NORMAL                       */
/*LINK = LOG                                      */
/*SCALE_DISP = DEVIANCE, PEARSON                  */
/**************************************************/

%if &base= %then %do;
	%put ERROR: You must be inform the dataset;
%end; %else %if &y= %then %do;
	%put ERROR: You must be inform the dependent variable;
%end; %else %if &x= %then %do;
	%put ERROR: You must be inform the covariates variables for mean model;
%end; %else %if &z= %then %do;
	%put ERROR: You must be inform the covariates variables for dispersion model;
%end; %else %do;

proc iml;
	MaxIter=&maxiter; 
	eps=&eps;
	Maxit=&maxit;
* Inserting Data;
	use &base;
		read all var{&y} into y[colname=depname];
		read all var{&x} into x0[colname=varname];
		read all var{&z} into z0[colname=dispname];
	close &base;
	
	
	X=x0;
	%if %upcase(&intercept)=Y %then %do;
	X=repeat(1,nrow(y),1)||x0;
	%end;

	Z=z0;
	%if %upcase(&intercept_disp)=Y %then %do;
	Z=repeat(1,nrow(y),1)||z0;
	%end;

	n=nrow(X);
	p=ncol(X)*ncol(Y);
	df=n-p;
	n1=nrow(Z);
	p1=ncol(Z)*ncol(Y);
	df1=n1-p1;

	alpha=&alpha;
	weight=repeat(1, n, 1);
	w=weight;


	
	%if %upcase(&dist)=BINOMIAL %then %do;
	tab=y||x||w;
	tabb=y||x||w;
	call sort(tabb,{1});
	freq=j(1,2,1); 
	do j=2 to nrow(tabb);
		if tabb[j,1]=tabb[j-1,1] then do;
		freq[nrow(freq),1]=tabb[j-1,1];
		freq[nrow(freq),2]=freq[nrow(freq),2]+1;
	end;
	else freq=freq//j(1,2,1);
	end;
	freq=(1:nrow(freq))`||freq;
	mi=1;
	y1=y;
	yd=j(nrow(y),nrow(freq)-1,0);
	do f=2 to nrow(freq);
		do ff=1 to nrow(y);
			if y[ff]=freq[f,2] then yd[ff,f-1]=1;
			else yd[ff,f-1]=0;
		end;
	end;
	y=yd;
	%end; %else %do;
	tab=y||x||w;
	%end;

*Mean GLM routine;
start MeanGLM;
	* Get initial values;
	%if %upcase(&link)=INV %then %do;
    yc = y + .000000000001; 
	y_trans = 1/yc;
	%end; %if %upcase(&link)=LOG %then %do;
	yc = y + .000000000001; 
	y_trans=log(yc);
	%end;
	%if %upcase(&link)=IDENTITY  %then %do;
	yc = y;  
	y_trans=yc;
	%end;
	%if %upcase(&link)=INV2  %then %do;
	yc = y + .000000000001;  
	y_trans = sqrt(1/yc);
	%end;
	%if %upcase(&link)=LOGIT  %then %do;
	yc=j(nrow(y), ncol(y),0);
	do f=1 to ncol(y); 
		yc[,f]=y[+,f]/n;
	end;
		y_trans = log(yc/(1-yc[,+]));
	%end;
	
	%if %upcase(&link)=INV2  %then %do;
		b=repeat(.000000000001,p,1);
	%end; %else %do;
	b=j(ncol(x), ncol(y),0);
	do ii=1 to ncol(y);
			b[,ii]=inv(X`*(X#weight))*X`*(y_trans[,ii]#weight);
	end; 
	%end;
	phi=1;
	eta=j(nrow(y), ncol(y),0);
	mu=j(nrow(y),ncol(y),0);
	si=j(ncol(y)*ncol(x), nrow(y),0);
	we=j(nrow(y), ncol(y),0);
	wo=j(nrow(y), ncol(y),0);
	H=j(ncol(y)*ncol(x), ncol(y)*ncol(x),0); 
	*IRLS algorithm Looping;
	do iter=1 to MaxIter until(abs((b-oldb)/oldb)<eps); 
		oldb=shapecol(b, ncol(x));
		do ii=1 to ncol(y);
			eta[,ii]=X*oldb[,ii];
		end;
		
		/*Link function*/
		%if %upcase(&link)=INV %then %do; 
		mu=1/eta;
		/*Firt derivative - link function*/
		gderiv=-1/mu##2;
		/*Second derivative - link function*/
		gderiv2=2/(mu##3);
		%end;
		%if %upcase(&link)=LOG %then %do; 
		mu=exp(eta);
		/*Firt derivative - link function*/
		gderiv=1/mu;
		/*Second derivative - link function*/
		gderiv2=-1/(mu##2);
		%end;
		%if %upcase(&link)=IDENTITY %then %do; 
		mu=eta;
		/*Firt derivative - link function*/
		gderiv=1;
		/*Second derivative - link function*/
		gderiv2=0;
		%end;
		%if %upcase(&link)=INV2 %then %do;
		eta=choose(eta<0,0.0000000000001,eta);
		mu=sqrt(1/eta);
		/*Firt derivative - link function*/
		gderiv=-2/(mu##3);
		/*Second derivative - link function*/
		gderiv2=6/(mu##4);
		%end;
		%if %upcase(&link)=LOGIT %then %do;
		do ii=1 to ncol(y);
			eta[,ii]=exp(X*oldb[,ii]);
		end;
		mu=mi#(eta/(1+eta[,+]));
		/*Firt derivative - link function*/
		gderiv=mi/(mu#(mi-mu));
		/*Second derivative - link function*/
		gderiv2=(mi#(2#mu-mi))/((mu#(mi-mu))##2);
		%end;
			
		%if %upcase(&dist)=GAMMA %then %do;
		/*Variance function*/	
		vmu=mu##2;
		/*Firt derivative - variance function*/
		vmuderiv=2#mu;
		%end; %if %upcase(&dist)=NORMAL %then %do;
		/*Variance function*/	
		vmu=1;
		/*Firt derivative - variance function*/
		vmuderiv=0;
		%end; %if %upcase(&dist)=POISSON %then %do;
		/*Variance function*/	
		vmu=mu;
		/*Firt derivative - variance function*/
		vmuderiv=1;
		%end; %if %upcase(&dist)=IG %then %do;
		/*Variance function*/	
		vmu=mu##3;
		/*Firt derivative - variance function*/
		vmuderiv=3#(mu##2);
		%end; %if %upcase(&dist)=BINOMIAL  %then %do;
		/*Variance function*/	
		vmu=(1/mi)#mu#(1-mu);
		/*Firt derivative - variance function*/
		vmuderiv=(1/mi)#(1-2#mu);
		%end; %if %upcase(&dist)=NB %then %do;
		/*Variance function*/	
		vmu=mu+k#(mu##2);
		/*Firt derivative - variance function*/
		vmuderiv=1+2#k#mu;
		%end;

		/*Gradient vector*/
		do ii=1 to ncol(y);
			m1=(ii-1)*ncol(x)+1;
			m2=ii*ncol(x);
			si[m1:m2,] = ((weight#(y[,ii]-mu[,ii]))/(vmu[,ii]#gderiv[,ii]#phi))`#X`;
		end;
		s = si[,+];
	
		/*Weights*/ 
		do ii=1 to ncol(y);
			we[,ii]=weight/(phi#vmu[,ii]#(gderiv[,ii]##2));
			wo[,ii]=we[,ii]+weight#(y[,ii]-mu[,ii])#((vmu[,ii]#gderiv2[,ii]+vmuderiv[,ii]#gderiv[,ii])/((vmu[,ii]##2)#(gderiv[,ii]##3)#phi));
		end;
		
		/*Hessian matrix*/
		do ii=1 to ncol(y);
			m1=(ii-1)*ncol(x)+1;
			m2=ii*ncol(x);
			H[m1:m2, m1:m2]=-(X#wo[,ii])`*X;
		end; 

		oldb=shapecol(oldb,0,1);
	
		b=oldb-inv(H)*s;
	end; 
finish;



	/*Dispersion model*/
	*Dispersion GLM routine;
start DispGLM;
	* Get initial values;
	%if %upcase(&link_disp)=LOG %then %do;
	ycd = ydisp + .000000000001; 
	y_transd=log(ycd);
	%end;
	q=j(ncol(z), ncol(ydisp),0);
	do ii=1 to ncol(ydisp);
			q[,ii]=inv(Z`*(Z#weightdisp))*Z`*(y_transd[,ii]#weightdisp);
	end; 
	phid=2;
	etad=j(nrow(ydisp), ncol(ydisp),0);
	mud=j(nrow(ydisp),ncol(ydisp),0);
	sid=j(ncol(ydisp)*ncol(z), nrow(y),0);
	wed=j(nrow(ydisp), ncol(ydisp),0);
	wod=j(nrow(ydisp), ncol(ydisp),0);
	Hd=j(ncol(ydisp)*ncol(z), ncol(ydisp)*ncol(z),0);
	*IRLS algorithm Looping;
	do iter=1 to MaxIter until(abs((q-oldq)/oldq)<eps);
		oldq=shapecol(q, ncol(z));
		do ii=1 to ncol(ydisp);
			etad[,ii]=Z*oldq[,ii];
		end;
		
		%if %upcase(&link_disp)=LOG %then %do; 
		mud=exp(etad);
		/*Firt derivative - link function*/
		gderivd=1/mud;
		/*Second derivative - link function*/
		gderiv2d=-1/(mud##2);
		%end;
		%if %upcase(&dist_disp)=GAMMA %then %do;
		/*Variance function*/	
		vmud=(mud##2);
		/*Firt derivative - variance function*/
		vmuderivd=2#mud;
		%end;
		

		/*Gradient vector*/
		do ii=1 to ncol(ydisp);
			m1=(ii-1)*ncol(z)+1;
			m2=ii*ncol(z);
			sid[m1:m2,] = ((weightdisp#(ydisp[,ii]-mud[,ii]))/(vmud[,ii]#gderivd[,ii]#phid))`#Z`;
		end;
		sd = sid[,+];
	
		/*Weights*/ 
		do ii=1 to ncol(ydisp);
			wed[,ii]=weightdisp/(phid#vmud[,ii]#(gderivd[,ii]##2));
			wod[,ii]=wed[,ii]+weightdisp#(ydisp[,ii]-mud[,ii])#((vmud[,ii]#gderiv2d[,ii]+vmuderivd[,ii]#gderivd[,ii])/((vmud[,ii]##2)#(gderivd[,ii]##3)#phid));
		end;
		
		/*Hessian matrix*/
		do ii=1 to ncol(ydisp);
			m1=(ii-1)*ncol(z)+1;
			m2=ii*ncol(z);
			Hd[m1:m2, m1:m2]=-(Z#wod[,ii])`*Z;
		end;

		oldq=shapecol(oldq,0,1);
	
		q=oldq-inv(Hd)*sd;
	end;
finish;


eqd=0;
start DGLM;
do iter=1 to Maxit until(abs((eqd-oldeqd)/(oldeqd+0.0000001))<0.0001);
oldeqd=eqd;
run MeanGLM;
	/*Deviance*/ 
	%if %upcase(&dist)=GAMMA %then %do; 
			di=2#(-log(yc/mu)+(yc-mu)/mu);
	%end; %if %upcase(&dist)=NORMAL %then %do;
			di=(y-mu)##2; 
	%end; %if %upcase(&dist)=POISSON %then %do;
			di=2#(yc#log(yc/mu)-(yc-mu));
	%end; %if %upcase(&dist)=IG %then %do;
			di=2#((yc-mu)##2)/(yc#mu##2);
	%end; %if %upcase(&dist)=BINOMIAL %then %do;
			di=mi#(yc#log(yc/mu))+(1-yc)#log((1-yc)/(1-mu));
	%end;
		
	/*Leverage*/
	hat=vecdiag((sqrt(we)#X)*ginv((X#we)`*X)*(X#sqrt(we))`); 
	
	/*Fitted dispersion e dispersion weight*/
	ydisp=((sign(y-mu)#sqrt(di))##2)/(1-hat);
	weightdisp=1-hat;

run DispGLM;
/*Predict and standard residual*/
	do ii=1 to ncol(ydisp);
		etad[,ii]=Z*q[,ii];
	end;

	preddisp=exp(etad);
	weight=1/preddisp;

	/*Extended quasi likelihood*/
	pi=constant('pi');
	eqllik = -ydisp/(2#preddisp) - 0.5#log(2#pi#preddisp#vmu);
	eqd= -2#sum(ydisp#eqllik); 
end;
finish;
run DGLM;
/***********MEAN MODEL**********************/
Report="%upcase(WORK.&base)"//"%upcase(&dist)"//"%upcase(&link)"//"&y";
ReportName={"Data Set" "Distribution" "Link Function" "Dependent Variable"};
print Report[label="Mean model Information" rowname=reportname];
				
			
%if %upcase(&scale)=DEVIANCE %then %do;
		%if %upcase(&dist)=GAMMA %then %do; 
			/*Deviance and Scale deviance*/
			deviance=2#sum(weight#(-log(yc/mu)+(yc-mu)/mu));
			scale1=deviance/df;
			/*scale1=pearson/((weight#fh)-p);*/
			scaled=1/scale1;
		%end; %if %upcase(&dist)=NORMAL %then %do;
			/*Deviance and Scale deviance*/
			deviance=sum(weight#(y-mu)##2);
			scale1=deviance/df;
			scaled=sqrt(scale1);
		%end; %if %upcase(&dist)=POISSON %then %do;
			/*Deviance and Scale deviance*/
			deviance=2#sum(weight#(yc#log(yc/mu)-(yc-mu)));
			scale1=deviance/df;
			scaled=sqrt(scale1);
		%end; %if %upcase(&dist)=IG %then %do;
			/*Deviance and Scale deviance*/
			deviance=sum(weight#((yc-mu)##2)/(yc#mu##2));
			scale1=deviance/df;
			scaled=sqrt(scale1);
		%end; %if %upcase(&dist)=BINOMIAL %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(weight#mi#(yc#log(yc/mu))+(1-yc)#log((1-yc)/(1-mu)));
			scale1=deviance/df;
			scaled=sqrt(scale1);
		%end; %if %upcase(&dist)=NB %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(yc#log(yc/mu)-(y+weight/k)#log((y+weight/k)/(mu+weight/k)));
			scale1=deviance/df;
			scaled=sqrt(scale1);
		%end;
		sdeviance=deviance/scale1;
	
		/*Pearson and Scale pearson*/
		pearson=sum((weight#(y-mu)##2)/vmu);
		scale2=pearson/df;
		spearson=pearson/scale1;

		valeudf=sdeviance/df;
		valeudf1=spearson/df;

		dev=df||deviance||scale1;
		sdev=df||sdeviance||valeudf;
		pcs=df||pearson||scale2;
		sp=df||spearson||valeudf1;

		phi=scale1;
	%end; %else %if  %upcase(&scale)=PEARSON %then %do;
		/*Pearson*/
		pearson=sum(((weight)#(y-mu)##2)/vmu);
		scale1=pearson/df;
		/*scale1=pearson/((weight#fh)-p);*/
		%if %upcase(&dist)=GAMMA %then %do; 
			/*Deviance and Scale pearson*/
			deviance=2#sum(weight#(-log(yc/mu)+(yc-mu)/mu));
			scale2=deviance/df;
			scaled=1/scale1;
			spearson=pearson/scale1;
		%end; %if %upcase(&dist)=NORMAL %then %do;
			/*Deviance and Scale pearson*/
			deviance=sum(weight#(y-mu)##2);
			scale2=deviance/df;
			scaled=sqrt(scale1);
			spearson=pearson/scale1;
		%end; %if %upcase(&dist)=POISSON %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(weight#(yc#log(yc/mu)-(yc-mu)));
			scale2=deviance/df;
			scaled=sqrt(scale1);
			spearson=pearson/scale1;
		%end; %if %upcase(&dist)=IG %then %do;
			/*Deviance and Scale pearson*/
			deviance=sum(weight#((yc-mu)##2)/(yc#mu##2));
			scale2=deviance/df;
			scaled=sqrt(scale1);
			spearson=pearson/scale1;
		%end; %if %upcase(&dist)=BINOMIAL %then %do;
			/*Deviance and Scale pearson*/
			deviance=2*sum(weight#mi#(yc#log(yc/mu))+(1-y)#log((1-yc)/(1-mu)));
			scale2=deviance/df;
			scaled=sqrt(scale1);
			spearson=pearson/scale1;
		%end; %if %upcase(&dist)=NB %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(yc#log(yc/mu)-(y+weight/k)#log((y+weight/k)/(mu+weight/k)));
			scale2=deviance/df;
			scaled=sqrt(scale1);
			spearson=pearson/scale1;
		%end;
		sdeviance=deviance/scale1;

		valeudf=sdeviance/df;
		valeudf1=spearson/df;

		dev=df||deviance||scale2;
		sdev=df||sdeviance||valeudf;
		pcs=df||pearson||scale1;
		sp=df||spearson||valeudf1;

		phi=scale1;
	%end;
	%else; %if %upcase(&scale)= %then %do;
	%if %upcase(&dist)=GAMMA %then %do; 
			/*Deviance and Scale deviance*/
			deviance=2#sum(weight#(-log(yc/mu)+(yc-mu)/mu));
			scale1=deviance/df;
		%end; %if %upcase(&dist)=NORMAL %then %do;
			/*Deviance and Scale deviance*/
			deviance=sum(weight#(y-mu)##2);
			scale1=deviance/df;
		%end; %if %upcase(&dist)=POISSON %then %do;
			/*Deviance and Scale deviance*/
			deviance=2#sum(weight#(yc#log(yc/mu)-(yc-mu)));
			scale1=deviance/df;
		%end; %if %upcase(&dist)=IG %then %do;
			/*Deviance and Scale deviance*/
			deviance=sum(weight#((yc-mu)##2)/(yc#mu##2));
			scale1=deviance/df;
		%end; %if %upcase(&dist)=BINOMIAL %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(weight#mi#(yc#log(yc/mu))+(1-yc)#log((1-yc)/(1-mu)));
			scale1=deviance/df;
		%end; %if %upcase(&dist)=NB %then %do;
			/*Deviance and Scale pearson*/
			deviance=2#sum(y#log(yc/mu)-(y+weight/k)#log((y+weight/k)/(mu+weight/k)));
			scale1=deviance/df;
		%end;
		sdeviance=deviance/scale1;
	
		/*Pearson and Scale pearson*/
		pearson=sum((weight#(y-mu)##2)/vmu);
		scale2=pearson/df;
		spearson=pearson/scale1;

		valeudf=sdeviance/df;
		valeudf1=spearson/df;

		dev=df||deviance||scale1;
		sdev=df||sdeviance||valeudf;
		pcs=df||pearson||scale2;
		sp=df||spearson||valeudf1;

	phi=1;
	scaled=1;
	%end;
	/*Weights*/ 
		do ii=1 to ncol(y);
			we[,ii]=weight/(phi#vmu[,ii]#(gderiv[,ii]##2));
			wo[,ii]=we[,ii]+weight#(y[,ii]-mu[,ii])#((vmu[,ii]#gderiv2[,ii]+vmuderiv[,ii]#gderiv[,ii])/((vmu[,ii]##2)#(gderiv[,ii]##3)#phi));
		end;
		
		/*Hessian matrix*/
		do ii=1 to ncol(y);
			m1=(ii-1)*ncol(x)+1;
			m2=ii*ncol(x);
			H[m1:m2, m1:m2]=-(X#wo[,ii])`*X;
		end;
	/*Covariance matrix*/
	Sigma=-inv(H);	
	std=sqrt(vecdiag(Sigma)); 

	/*Wald confidence limits*/
	lower=b-quantile("Normal", 1-alpha/2)#std;
	upper=b+quantile("Normal", 1-alpha/2)#std;
	limits=lower||upper;


	/*Wald Chi-square*/
	wald=(b/std)##2;
	pvalue=1-probchi(wald, 1);

	*b1=b`||scaled;
	b1=shapecol(b,0,1)`;
	b2=b1`;
	
	varname1=varname`;
	%if %upcase(&intercept)=Y %then %do; 
	varname1="Intercept"//varname`;
	%end;
	
	print "Mean Model Analysis of Likelihood Estimates";

	print varname1[label="Parameter"] 
		  b2[label="Estimate" format=12.4] std[label="Standard Error" format=12.4] limits[label="Wald 95% Confidence Limits" format=12.4] wald[label="Wald Chi-Square" format=12.2] 
		  pvalue[label="Pr > ChiSq" format=pvalue6.4];
	
	%if %upcase(&scale)=DEVIANCE %then %do;
		%if %upcase(&dist)=GAMMA %then %do;
			print "Note: The Gamma scale parameter was estimated by DOF/DEVIANCE.";
		%end; %if %upcase(&dist)=NORMAL | %upcase(&dist)=POISSON | %upcase(&dist)=IG | %upcase(&dist)=BINOMIAL | %upcase(&dist)=NB %then %do;
			print "Note: The scale parameter was estimated by the square root of DEVIANCE/DOF.";
		%end;
	%end; %if  %upcase(&scale)=PEARSON %then %do;
	%if %upcase(&dist)=GAMMA %then %do;
			print "Note: The Gamma scale parameter was estimated by  DOF/Pearson's Chi-Square.";
		%end; %if %upcase(&dist)=NORMAL | %upcase(&dist)=POISSON | %upcase(&dist)=IG | %upcase(&dist)=BINOMIAL | %upcase(&dist)=NB %then %do;
			print "Note: The scale parameter was estimated by the square root of Pearson's Chi-Square/DOF. ";
		%end;
	%end;

	/***********DISPERSION MODEL**********************/
Report="%upcase(&dist_disp)"//"%upcase(&link_disp)";
ReportName={"Distribution" "Link Function"};
print Report[label="Dispersion model Information" rowname=reportname];
%if %upcase(&scale_disp)=DEVIANCE %then %do;
		%if %upcase(&dist_disp)=GAMMA %then %do; 
			/*Deviance and Scale deviance*/
			deviance=2#sum(weightdisp#(-log(ydisp/mud)+(ydisp-mud)/mud));
			scale1=deviance/df1;
			scaled=1/scale1;
		%end; %if %upcase(&dist_disp)=NORMAL %then %do;
			/*Deviance and Scale deviance*/
			deviance=sum(weightdisp#(ydisp-mud)##2);
			scale1=deviance/df1;
			scaled=sqrt(scale1);
		%end;
		sdeviance=deviance/scale1;
	
		/*Pearson and Scale pearson*/
		pearson=sum((weightdisp#(ydisp-mud)##2)/vmud);
		scale2=pearson/df1;
		spearson=pearson/scale1;

		valeudf=sdeviance/df;
		valeudf1=spearson/df;

		dev=df||deviance||scale1;
		sdev=df||sdeviance||valeudf;
		pcs=df||pearson||scale2;
		sp=df||spearson||valeudf1;

		phid=scale1;
	%end; 
	/*Weights*/ 
		do ii=1 to ncol(ydisp);
			wed[,ii]=weightdisp/(phid#vmud[,ii]#(gderivd[,ii]##2));
			wod[,ii]=wed[,ii]+weightdisp#(ydisp[,ii]-mud[,ii])#((vmud[,ii]#gderiv2d[,ii]+vmuderivd[,ii]#gderivd[,ii])/((vmud[,ii]##2)#(gderivd[,ii]##3)#phid));
		end;
		
		/*Hessian matrix*/
		do ii=1 to ncol(ydisp);
			m1=(ii-1)*ncol(z)+1;
			m2=ii*ncol(z);
			H[m1:m2, m1:m2]=-(Z#wod[,ii])`*Z;
		end;
	/*Covariance matrix*/
	Sigmad=-inv(Hd);	
	stdd=sqrt(vecdiag(Sigmad)); 

	/*Wald confidence limits*/
	lower=q-quantile("Normal", 1-alpha/2)#stdd;
	upper=q+quantile("Normal", 1-alpha/2)#stdd;
	limits=lower||upper;


	/*Wald Chi-square*/
	wald=(q/stdd)##2;
	pvalue=1-probchi(wald, 1);

	*b1=b`||scaled;
	b1=shapecol(q,0,1)`||scaled;
	b2=b1`;
	
	varname1=dispname`//"Scale";
	%if %upcase(&intercept_disp)=Y %then %do; 
	varname1="Intercept"//dispname`//"Scale";
	%end;
	
	print "Dispersion Model Analysis of Likelihood Estimates";
	print varname1[label="Parameter"] 
		  b2[label="Estimate" format=12.4] stdd[label="Standard Error" format=12.4] limits[label="Wald 95% Confidence Limits" format=12.4] wald[label="Wald Chi-Square" format=12.2] 
		  pvalue[label="Pr > ChiSq" format=pvalue6.4];
	
	%if %upcase(&scale_disp)=DEVIANCE %then %do;
		%if %upcase(&dist_disp)=GAMMA %then %do;
			print "Note: The Gamma scale parameter was estimated by DOF/DEVIANCE.";
		%end; %if %upcase(&dist_disp)=NORMAL | %upcase(&dist_disp)=POISSON | %upcase(&dist_disp)=IG | %upcase(&dist_disp)=BINOMIAL %then %do;
			print "Note: The scale parameter was estimated by the square root of DEVIANCE/DOF.";
		%end;
	%end; %if  %upcase(&scale_disp)=PEARSON %then %do;
	%if %upcase(&dist_disp)=GAMMA %then %do;
			print "Note: The Gamma scale parameter was estimated by  DOF/Pearson's Chi-Square.";
		%end; %if %upcase(&dist_disp)=NORMAL | %upcase(&dist_disp)=POISSON | %upcase(&dist_disp)=IG | %upcase(&dist_disp)=BINOMIAL %then %do;
			print "Note: The scale parameter was estimated by the square root of Pearson's Chi-Square/DOF. ";
		%end;
	%end;

print maxit[label="Number of iterations"];

quit;
%end;
%mend doubleglm;

/*EXAMPLE 1*/
data base1;
do i=1 to 1000;
x=rannor(2);
mu=x+3;
z=rannor(3);
sigma2=exp(-2*z+1);
y=rannor(4)*sqrt(sigma2)+mu;
output;
end;
run;

%doubleglm(base=base1, y=y, x=x, z=z, dist=normal, scale=deviance);

/*EXAMPLE 2*/
data base2;
do i=1 to 1000;
x=rannor(2);
mu=x;
z=rannor(3);
sigma2=exp(-2*z);
y=rannor(4)*sqrt(sigma2)+mu;
output;
end;
run;

%doubleglm(base=base2, y=y, x=x, z=z, dist=normal, intercept=N, intercept_disp=N, scale=deviance);
